@@ -467,6 +467,8 @@ data EventInfo
                        , tickyCtrDefArity   :: !Word16
                        , tickyCtrDefKinds   :: !Text
                        , tickyCtrDefName    :: !Text
+                       , tickyCtrInfoTbl    :: !Word64
+                       , tickyCtrJsonDesc   :: Maybe Text
                        }
   | TickyCounterSample
                        { tickyCtrSampleId         :: !Word64
