@@ -73,7 +73,6 @@ import Control.Applicative
 import Control.Concurrent hiding (ThreadId)
 import qualified Data.Binary.Put as P
 import qualified Data.ByteString as B
-import qualified Data.ByteString.Char8 as B8
 import qualified Data.ByteString.Lazy as BL
 import Data.Char (isPrint)
 import Data.IntMap (IntMap)
@@ -529,6 +528,7 @@ buildEventInfo spec' =
           <> ", " <>  "arity: " <> TB.decimal tickyCtrDefArity
           <> ", " <> "def kinds: " <> TB.fromText tickyCtrDefKinds
           <> ", " <> "name: " <> TB.fromText tickyCtrDefName
+          <> ", " <> "itbl: " <> TB.hexadecimal tickyCtrInfoTbl
         TickyCounterSample {..}  ->
           "ticky counter sample " <> TB.decimal tickyCtrSampleId
           <> ": " <> "entry count: " <> TB.decimal tickyCtrSampleEntryCount
@@ -542,7 +542,7 @@ replaceUnprintableWith
   :: Char -- ^ Replacement character
   -> B.ByteString -- ^ Binary message which may contain unprintable bytes
   -> T.Text
-replaceUnprintableWith replacement = TE.decodeUtf8 . B8.map replace
+replaceUnprintableWith replacement = T.map replace . TE.decodeUtf8With (\_ _ -> Just replacement) 
   where
     replace c
       | isPrint c = c
