@@ -2,13 +2,9 @@ module Utils where
 
 files :: [FilePath]
 files = map ("test/"++)
-    [ "queens-ghc-6.12.1.eventlog"
-    , "queens-ghc-7.0.2.eventlog"
-    , "mandelbrot-mmc-2011-06-14.eventlog"
-    , "nonmoving-gc.eventlog"
+    [ "nonmoving-gc.eventlog"
     , "nonmoving-gc-census.eventlog"
     , "parallelTest.eventlog"
-    , "pre77stop.eventlog", "782stop.eventlog", "783stop.eventlog"
     , "sleep.h.eventlog"
     , "sleep.hC.eventlog"
     , "sleep.hm.eventlog"
@@ -19,9 +15,12 @@ files = map ("test/"++)
     , "biographical-samples.eventlog"
     , "time-prof.eventlog"
     , "trace-binary-event.eventlog"
+    , "trace-binary-nonutf.eventlog"
     , "unicode.eventlog"
     , "ticky-ticky.eventlog"
     , "ticky-begin-sample.eventlog"
+    , "ticky-new.eventlog"
+    , "ticky-json.eventlog"
     ]
 
 
